//Raspberry Pi Scratch2用 ESPr Developer拡張機能
//対応したファームウェアが必要です
// http://tsq.gapj.net

(function(ext) {

    //
    var MYESPR_IP = '192.168.4.1';
    //ESPr DeveloperのURL
    var ESPR_URL = 'http://' + MYESPR_IP;
    //ESPr Developerのポート番号
    const ESPR_PORT_AT = '80';

    var poll_state = '';
    var tempreture = '';
    var pressure = '';
    var report_string = '';
    var analogPins = [];
    var digitalPins = [];

    var BASEURL = ESPR_URL + ':' + ESPR_PORT_AT + '/';

    ext._shutdown = function() {};

    ext._getStatus = function() {
        if(poll_state == 'good'){
            return {status: 2, msg: 'Ready'};
        }
        else {
            return {status: 1, msg: 'Not Ready'};
        }
    };

    //Polling(100msec)
    setInterval( function() {
        $.ajax({
              url: BASEURL + 'POLL',
              dataType: 'text',
        }).done( (data) => {
            retVals = data.split(':');
            retVals.forEach(function(retdata){
              var tmp_str  = retdata.split(' ');
              //気温
              if(tmp_str[0].charAt(0) == 'T'){
		tempreture = tmp_str[1];
              }  
              //気圧
              if(tmp_str[0].charAt(0) == 'P'){
		pressure = tmp_str[1];
              }  
              if(tmp_str[0].charAt(0) == 'D'){
		digitalPins[tmp_str[0]] = tmp_str[1];
              }  
              if(tmp_str[0].charAt(0) == 'A'){
		analogPins[tmp_str[0]] = tmp_str[1];
              }  
            });
            poll_state = 'good';
	}).fail( (data) => {
            poll_state = 'bad';
        });
    }, 100);

    ext.tempreture = function() {
        return tempreture;
    };

    ext.pressure = function() {
	return pressure;
    };

    ext.pinmode = function(pin, val) {
	var tmp_pin = pin.replace('IO', 'M');
	if(val == 'OUTPUT'){ var tmp_val='1'}
	if(val == 'INPUT'){ var tmp_val='0'}
        var TARGET = BASEURL +  'PINCTRL?pmode=' + tmp_pin + '-' + tmp_val;
        $.ajax({
              url: TARGET,
              dataType: 'html',
	})
        return {status: 2, msg: 'Ready'};
    };

    ext.pinval = function(pin, val) {
	var tmp_pin = pin.replace('IO', 'D');
	if(val == 'High'){ var tmp_val='1'}
	if(val == 'Low'){ var tmp_val='0'}
        var TARGET = BASEURL + 'PINCTRL?pmode='  + tmp_pin + '-' + tmp_val;
        $.ajax({
              url: TARGET,
              dataType: 'html',
	})
        return {status: 2, msg: 'Ready'};
    };

    ext.pinpwm = function(pin, val) {
	var tmp_pin = pin.replace('IO', 'A');
        var TARGET = BASEURL + 'PINCTRL?pmode='  + tmp_pin + '-' + val;
        $.ajax({
              url: TARGET,
              dataType: 'html',
	})
        return {status: 2, msg: 'Ready'};
    };

    ext.servoPin = function(servo, pin) {
        var tmp_pin = pin.slice(2);
        var tmp_servo = parseInt(servo) + parseInt('11');
        var TARGET = BASEURL + 'PINCTRL?pmode=U'  + tmp_servo + '-' + tmp_pin;
        $.ajax({
              url: TARGET,
              dataType: 'html',
	})
        return {status: 2, msg: 'Ready'};
    };

    ext.servoAngle = function(servo, val) {
        var tmp_servo = parseInt(servo) + parseInt('11');
        var TARGET = BASEURL + 'PINCTRL?pmode=V'  + tmp_servo + '-' + val;
        $.ajax({
              url: TARGET,
              dataType: 'html',
	})
        return {status: 2, msg: 'Ready'};
    };

    ext.analogValue = function(pin) {
        return analogPins['A0'];
    };

    ext.digitalValue = function(pin) {
	var tmp_pin = pin.replace('IO', 'D');
        return digitalPins[tmp_pin];
    };

    ext.gettemppress = function() {
        $.ajax({
              url: BASEURL + 'TEMPP',
              dataType: 'html',
	})
        return {status: 2, msg: 'Ready'};
    };

    ext.setEsprIp = function(addr) {
	if(addr != ''){
	    MYESPR_IP = addr;
	    ESPR_URL = 'http://' + MYESPR_IP;
	    BASEURL = ESPR_URL + ':' + ESPR_PORT_AT + '/';
	}
        return {status: 2, msg: 'Ready'};
    };

    ext.getEsprIp = function() {
        return MYESPR_IP;
    };


    // Block and block menu descriptions
    var descriptor = {
	blocks: [
		[" ", " %m.digitalPins ピンを %m.pinModes にする", "pinmode", "IO12", "INPUT"],
		[" ", " %m.digitalPins ピンを %m.pinValues にする", "pinval", "IO12", "Low"],
		[" ", " %m.pwmPins ピンに %n を出力する(PWM)", "pinpwm", "IO12", "0"],
		["r", "デジタル %m.digitalPins の値", "digitalValue","IO12"],
		["r", "アナログA0の値","analogValue","A0"],
		[" ", "気温気圧を測定", "gettemppress"],
		['r', "気温", 'tempreture'],
		["r", "気圧", "pressure"],
		[" ", "サーボ %m.servonum を %m.digitalPins に接続する", "servoPin", "1", "IO12"],
		[" ", "サーボ %m.servonum の角度を %n にする", "servoAngle", "1", "0"],
		[" ", "自分のESPrを %s にする", "setEsprIp", "192.168.4.1"],
		["r", "自分のESPr", "getEsprIp"],
	],
	menus: {
		"digitalPins": ["IO12","IO13","IO14","IO15","IO16"],
		"pwmPins": ["IO12","IO13","IO14","IO15","IO16"],
		"pinValues": ["Low", "High"],
		"pinModes": ["INPUT", "OUTPUT"],
		"servonum": ["1", "2", "3", "4", "5"],
	},
    };

    ScratchExtensions.register('ESPr Developer', descriptor, ext);
})({});
